#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.ServiceModel;
using HeuristicLab.Hive.Contracts.BusinessObjects;
using HeuristicLab.Hive.Contracts.ResponseObjects;
using HeuristicLab.PluginInfrastructure;

namespace HeuristicLab.Hive.Contracts.Interfaces {
  /// <summary>
  /// This is the facade for the client communication
  /// </summary>
  [ServiceContract]
  public interface ISlaveCommunicator {

    /// <summary>
    /// This method registers the slave and marks it as online
    /// </summary>
    [OperationContract]
    Response Login(SlaveDto slave);

    [OperationContract]
    ResponseHeartBeat ProcessHeartBeat(HeartBeatData hbData);

    [OperationContract]
    ResponseObject<JobDto> GetJob(Guid clientId);

    [OperationContract]
    ResponseResultReceived StoreFinishedJobResult(Guid clientId, Guid jobId, byte[] result, TimeSpan executionTime, string exception);

    [OperationContract]
    ResponseResultReceived ProcessSnapshot(Guid clientId, Guid jobId, byte[] result, TimeSpan executionTime, string exception);

    [OperationContract]
    Response Logout(Guid clientId);

    [OperationContract]
    Response IsJobStillNeeded(Guid jobId);

    [OperationContract]
    ResponseList<CachedHivePluginInfoDto> GetPlugins(List<HivePluginInfoDto> pluginList);

    [OperationContract]
    ResponseCalendar GetCalendar(Guid clientId);

    [OperationContract]
    Response SetCalendarStatus(Guid clientId, CalendarState state);

    ResponseObject<HivePluginFile> GetConfigurationFile();
  }
}
