﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.ServiceModel;
using HeuristicLab.Clients.Common;
using HeuristicLab.Common;

namespace HeuristicLab.Hive.Contracts {
  public class WcfServicePool<T> where T : class {
    private static object locker = new object();

    private string hostAddress;
    public string HostAddress {
      get { return hostAddress; }
      set { hostAddress = value; }
    }
    private string userName;
    public string UserName {
      get { return userName; }
      set { userName = value; }
    }
    private string password;
    public string Password {
      get { return password; }
      set { password = value; }
    }

    private string endpointName;

    public WcfServicePool(string endpointName) {
      this.endpointName = endpointName;
    }
    public WcfServicePool(string hostAddress, string endpointName) : this(endpointName) {
      this.hostAddress = hostAddress;
    }
    public WcfServicePool(string endpointName, string userName, string password) : this(endpointName) {
      this.userName = userName;
      this.password = password;
    }

    public Disposable<T> GetService() {
      try {
        if (!string.IsNullOrEmpty(userName) && !string.IsNullOrEmpty(password)) {
          var factory = ClientFactory.CreateChannelFactory<T>(endpointName, hostAddress, userName, password);
          return new Disposable<T>(factory.Obj.CreateChannel());
        } else {
          var factory = ClientFactory.CreateChannelFactory<T>(endpointName, hostAddress);
          return new Disposable<T>(factory.Obj.CreateChannel());     
        }
      }
      catch (EndpointNotFoundException ex) {
        OnExceptionOccured(ex);
      }
      return null;
    }

    public event EventHandler<EventArgs<Exception>> ExceptionOccured;
    private void OnExceptionOccured(Exception e) {
      lock (locker) {
        var handler = ExceptionOccured;
        if (handler != null) handler(this, new EventArgs<Exception>(e));
      }
    }
  }
}
